#====================== BEGIN GPL LICENSE BLOCK ======================
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#======================= END GPL LICENSE BLOCK ========================

# Usage:
# import bpy
# from . import [name]_ui
# from .libs import addon_prefs


# class [Name]AddonPreferences(addon_prefs.BaseAddonPreferences):
#     bl_idname = __package__

#     addon_name = 'wr_[name]'
#     module_ui = [name]_ui
#     panel = [name]_ui.[NAME]_PT_main



# ################################################################
# # Register 
# def register():
#     addon_prefs.register([Name]AddonPreferences)


# def unregister():
#     addon_prefs.unregister([Name]AddonPreferences)

# Open Pref from UI:
# row.operator(ADDON_OT_open_prefs.bl_idname, icon='SETTINGS', text='')



import bpy
from bpy.app.handlers import persistent
from . import tools
# from .. import bl_info



class BaseAddonPreferences(bpy.types.AddonPreferences):
    bl_idname = __package__

    # Usage:
    # addon_name = 'wr_pbr_bridge'
    # module_ui = pbr_bridge_ui
    # panel = pbr_bridge_ui.PBR_BRIDGE_PT_main


    @persistent
    def update_addon_category_reg(none):
        try:
            cat = bpy.context.preferences.addons[BaseAddonPreferences.child_class.addon_name].preferences.addon_category
            BaseAddonPreferences.child_class.panel.bl_category = cat
        except:
            return

        BaseAddonPreferences.child_class.module_ui.unregister()
        BaseAddonPreferences.child_class.module_ui.register()


    def update_addon_category(self, context):
        BaseAddonPreferences.update_addon_category_reg(None)


    addon_category: bpy.props.StringProperty(
        default = 'iwHiteRabbiT',
        update = update_addon_category,
    )


    def draw(self, context):
        layout = self.layout
        row = layout.row()
        row.prop(self, 'addon_category', text='N-Panel name')



class ADDON_OT_open_prefs(bpy.types.Operator):
    bl_idname = "addon.open_prefs"
    bl_label = "Open addon preferences"
    bl_options = {'REGISTER', 'UNDO'}

    addon_name : bpy.props.StringProperty(default='')

    def execute(self, context):
        if not self.addon_name:
            tools.show_message_box('Addon name missing.', 'Missing argument.', 'ERROR')
            return {'CANCELLED'}

        bpy.context.preferences.active_section = 'EXTENSIONS'
        bpy.data.window_managers["WinMan"].addon_search = self.addon_name #bl_info['name']
        bpy.ops.screen.userpref_show('INVOKE_DEFAULT')        

        self.addon_name = ''

        return {'FINISHED'}



class ADDON_OT_open_keymaps(bpy.types.Operator):
    bl_idname = "addon.open_keymaps"
    bl_label = "Open addon keymaps"
    bl_options = {'REGISTER', 'UNDO'}

    filter : bpy.props.StringProperty(default='')

    def execute(self, context):
        if not self.filter:
            tools.show_message_box('Filter missing.', 'Missing argument.', 'ERROR')
            return {'CANCELLED'}

        bpy.context.preferences.active_section = 'KEYMAP'
        bpy.ops.screen.userpref_show('INVOKE_DEFAULT')
        bpy.data.screens['temp'].areas[0].spaces[0].filter_text = self.filter

        self.filter = ''

        return {'FINISHED'}



################################################################
# Register 
def register(child_class):
    tools.register([ADDON_OT_open_prefs, ADDON_OT_open_keymaps, child_class])

    BaseAddonPreferences.child_class = child_class

    bpy.app.handlers.version_update.append(child_class.update_addon_category_reg)


def unregister(child_class):
    tools.unregister([ADDON_OT_open_prefs, ADDON_OT_open_keymaps, child_class])

    bpy.app.handlers.version_update.remove(child_class.update_addon_category_reg)
