#====================== BEGIN GPL LICENSE BLOCK ======================
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#======================= END GPL LICENSE BLOCK ========================



import bpy
from . import tools



class YIELD_OT_base(bpy.types.Operator):
    # bl_label = "Edit"
    # bl_options = {'REGISTER', 'UNDO'}

    static_step = None
    generator = None
    progress_bar = None
    _timer = None


    def get_generator(self):
        raise NotImplementedError()

    
    def get_progress_bar(self):
        raise NotImplementedError()


    steps = 1
    cur_step = 0
    @classmethod
    def step(cls, msg):
        if cls.steps - 1 == 0:
            cls.cur_step = cls.cur_step + 1
            return 0.5, msg

        pc = (cls.cur_step * 100) // (cls.steps - 1)
        cls.cur_step = cls.cur_step + 1
        return pc, msg


    def modal(self, context, event):

        if event.type == 'ESC':
            self.end(context, True)
            return {'CANCELLED'}

        if self.generator is None:
            self.generator = self.get_generator()

        if self.progress_bar is None:
            self.progress_bar = self.get_progress_bar()

        try:
            for step in self.generator:
                YIELD_OT_base.static_step = step
                self.progress_bar = step[0]

                return {'RUNNING_MODAL'}

        except Exception as e:
            bpy.ops.wm.dialog('INVOKE_DEFAULT',
                message=f'{e}',
                title='Error during operation',
                icon='ERROR')

            import traceback
            # print(e.__traceback__)
            print(traceback.format_exc())

            self.end(context, False)
            return {'FINISHED'}

        self.end(context)
        return {'FINISHED'}


    def end(self, context, do_undo=False):
        context.window_manager.event_timer_remove(self._timer)
        self.generator = None
        YIELD_OT_base.static_step = None
        if hasattr(context.area, 'tag_redraw'):
            context.area.tag_redraw()

        if do_undo:
            # Undo anything done by this OP
            bpy.app.timers.register(bpy.ops.ed.undo, first_interval=0.001)


    def execute(self, context):
        wm = context.window_manager
        self._timer = wm.event_timer_add(time_step=0.1, window=context.window)
        context.window_manager.modal_handler_add(self)  
        return {'RUNNING_MODAL'}


