#====================== BEGIN GPL LICENSE BLOCK ======================
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#======================= END GPL LICENSE BLOCK ========================


import bpy
from .libs import tools



class PHYXDRAGProperty(bpy.types.PropertyGroup):
    Collide: bpy.props.BoolProperty(
        default=False,
        override={'LIBRARY_OVERRIDABLE'}
    )

    Shape: bpy.props.EnumProperty(
        items=[
            ('BOX', 'Box', 'Box-like shapes (i.e. cubes), including planes (i.e. ground planes)'),
            ('SPHERE','Sphere','Sphere'),
            ('CAPSULE','Capsule','Capsule'),
            ('CYLINDER','Cylinder','Cylinder'),
            ('CONE','Cone','Cone'),
            ('CONVEX_HULL','Convex Hull','A mesh-like surface encompassing (i.e. shrinkwrap over) all vertices (best results with fewer vertices)'),
            ('MESH','Mesh','Mesh consisting of triangles only, allowing for more detailed interactions than convex hulls'),
            ('COMPOUND','Compound Parent','Combines all of its direct rigid body children into one rigid object'),
        ],
        default='CONVEX_HULL',
        override={'LIBRARY_OVERRIDABLE'}
    )



class PHYXDRAGGenProperty(bpy.types.PropertyGroup):
    substeps_per_frame: bpy.props.IntProperty(
        name='Substeps Per Frame',
        default=10,
        min=1
    )

    solver_iterations: bpy.props.IntProperty(
        name='Solver Iterations',
        default=10,
        min=10
    )

    collision_margin: bpy.props.FloatProperty(
        name='Collision Margin',
        default=0,
        min=0,
        unit='LENGTH'
    )



################################################################
# Register 
classes = [
    PHYXDRAGProperty,
    PHYXDRAGGenProperty,
]


def register():
    tools.register(classes)

    bpy.types.Object.PhyXDrag = bpy.props.PointerProperty(name='PhyXDrag', type=PHYXDRAGProperty, override={'LIBRARY_OVERRIDABLE'})
    bpy.types.Scene.PhyXDragGen = bpy.props.PointerProperty(name='PhyXDragGen', type=PHYXDRAGGenProperty, override={'LIBRARY_OVERRIDABLE'})


def unregister():
    tools.unregister(classes)
