#====================== BEGIN GPL LICENSE BLOCK ======================
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#======================= END GPL LICENSE BLOCK ========================


import bpy
from .libs import tools
from .phyxdrag_operators import *
from .libs.addon_prefs import ADDON_OT_open_prefs, ADDON_OT_open_keymaps
from . import bl_info


key_desc = '''Default Key 'F'
'F': drag with physics
'shift': repulse/attract objects
'ctrl': affect only active object
'alt': limit to vertical force'''



class PHYXDRAGPanel_Base():
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "iwHiteRabbiT"
    bl_options = {"DEFAULT_CLOSED"}

    poll_context = ('OBJECT', 'POSE')
    

    @classmethod
    def poll(cls, context):
       return context.mode in cls.poll_context



class PHYXDRAG_PT_tool_panel(PHYXDRAGPanel_Base, bpy.types.Panel):
    bl_idname = "PHYXDRAG_PT_tool_panel"
    bl_label = bl_info['name']


    def draw_header_preset(self, context):
        layout = self.layout
        op = layout.operator(ADDON_OT_open_prefs.bl_idname, icon='SETTINGS', text='')        
        op.addon_name = bl_info['name']


    def draw(self, context):
        layout = self.layout

    #     op = layout.operator(ADDON_OT_open_keymaps.bl_idname, icon='SETTINGS', text='Keymap')        
    #     op.filter = 'phyxdrag'



class PHYXDRAG_PT_object(PHYXDRAGPanel_Base, bpy.types.Panel):
    bl_parent_id = "PHYXDRAG_PT_tool_panel"
    bl_label = "Object"
    bl_options = {"HEADER_LAYOUT_EXPAND"}

    @classmethod
    def poll(cls, context):
        p = super(PHYXDRAG_PT_object, cls).poll(context)

        if bpy.context.object:
            c = bpy.context.object.type == 'MESH'
        else:
            c = False
        
        return p and c


    # def draw_header(self, context):
    #     layout = self.layout
    #     layout.prop(bpy.context.object, 'name')


    def draw_header_preset(self, context):
        if bpy.context.object:
            layout = self.layout
            layout.alert = True
            layout.label(text=bpy.context.object.name)
    

    def draw(self, context):
        layout = self.layout

        obj = bpy.context.object

        if obj and hasattr(obj, 'PhyXDrag'):
            row = layout.row()
            row.prop(obj.PhyXDrag, 'Collide')

            row = layout.row()
            row.prop(obj.PhyXDrag, 'Shape')



class PHYXDRAG_PT_options(PHYXDRAGPanel_Base, bpy.types.Panel):
    bl_parent_id = "PHYXDRAG_PT_tool_panel"
    bl_label = "Options"
    # bl_options = {"HEADER_LAYOUT_EXPAND"}


    def draw(self, context):
        layout = self.layout

        row = layout.row()
        row.prop(context.scene.PhyXDragGen, 'collision_margin')

        tools.label_multiline(context, layout, key_desc, icon='INFO', first_line_red=False)
        op = layout.operator(ADDON_OT_open_keymaps.bl_idname, icon='SETTINGS', text='Keymap')        
        op.filter = 'phyxdrag'

        tools.label_multiline(context, layout, "If you're experimenting odd behavior with constraints, increase both of these values to at least 100.", icon='INFO', first_line_red=False)

        row = layout.row()
        row.prop(context.scene.PhyXDragGen, 'substeps_per_frame')

        row = layout.row()
        row.prop(context.scene.PhyXDragGen, 'solver_iterations')        



################################################################
# Register
classes = [
    PHYXDRAG_PT_tool_panel,
    PHYXDRAG_PT_object,
    PHYXDRAG_PT_options,
]

         
def register():
    tools.register(classes)


def unregister():
    tools.unregister(classes)
